#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>


/*
*    Lookup telephone number
*    Required external program is w3m.
*
*    Author: R.J. van der Putten, Leiden, Holland, 
*    rob at sput dot nl.
* 
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License version 3 as
*    published by the Free Software Foundation.
* 
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*/


/*  w3m and Url  */
#define W3M_MAX_TM 3
#define W3M_COMND  "w3m -cols 240 -o display_charset=UTF-8 -o ssl_verify_server=n -dump"
#define URL_PREFIX "http://www.nummerid.com/result.php?input_telefoon="
#define URL_SUFFIX "&submit_telefoon.x=52&submit_telefoon.y=18&input_naam=&input_adres=&input_postcode="

int cont;

/*  Kill aftre timeout  */
void timeout(int sig)
{
    cont = 0;
}

/*  Find name in line  */
void findname(char *num, char *line, int len)
{
    int i, j;

    i = 0;
    j = 0;
    while(i < len) {
	if (strncmp(line + i, "  ", 2) == 0) {
	    /*  Wide whitespace  */
	    i += 2;
	    break;
	}
	i++;
    }
    while (i < len) {
	if (line[i] > ' ' || line[i] < 0) {
	    /*  End of whitespace  */
	    j = i;
	    break;
	}
	i++;
    }
    while (i < len) {
	if (strncmp(line + i, "  ", 2) == 0) {
	    /*  Wide whitespace  */
	    line[i] = 0;
	    if(j > 0 && j < i)
		printf("%s\t%s\n", num, line + j);
	    break;
	}
	i++;
    }
}


int main(int argc, char **argv)
{
    char buf[4096], comnd[256], line[4096];
    int cont, i, inbuf, inp, len, outp;
    FILE *p1;

    i = 0;
    inbuf = 0;
    inp = 0;
    len = 0;
    outp = 0;
    memset(buf, 0, 4096);
    memset(comnd, 0, 256);
    memset(line, 0, 4096);
    p1 = NULL;

    cont = 1;		/*  Continue if not zero  */

    if (argc != 2) {
	fprintf(stderr, "Usage: namelookup number\n");
	exit(1);
    }

    /*  Check for number; +0123456789  */
    i = 0;
    len = strlen(argv[1]);
    while (i < len) {
	if ((argv[1][i] < '0' || argv[1][i] > '9') && argv[1][i] != '+') {
	    fprintf(stderr, "Not a number\n");
	    exit(1);
	}
	i++;
    }

    /*  Check for starts with '+'  */
    if (argv[1][0] == '+') {
	/*  Escape '+' to '%2B'  */
	snprintf(comnd, 255, "%s \"%s%%2B%s%s\"", \
	  W3M_COMND, URL_PREFIX, argv[1] + 1, URL_SUFFIX);
    } else {
	snprintf(comnd, 255, "%s \"%s%s%s\"", \
	  W3M_COMND, URL_PREFIX, argv[1], URL_SUFFIX);
    }

    /*  Set timeout  */
    signal(SIGALRM, timeout);
    alarm(W3M_MAX_TM);

    /*  Open process  */
    p1 = popen(comnd, "r");
    if (p1 == NULL) {
	fprintf(stderr, "Can't open w3m\n");
	exit(1);
    }

    /*
    *  Read process
    *  Html page may be > 4095 bytes
    */
    while ((inbuf = fread(buf, 1, 4095, p1)) > 0 && cont != 0) {
	inp = 0;
	while (inp < inbuf) {
	    line[outp] = buf[inp];
	    if (line[outp] == 10 || outp > 4092) {
		/*  A line or lots of data  */
		outp++;
		line[outp] = 0;
		if (line[0] == '+') {
		    /*  Number  */
		    findname(argv[1], line, outp);
		}
		outp = 0;
		line[outp] = 0;
	    } else {
		outp++;
	    }
	    inp++;
	}
    }
    alarm(0);
    signal(SIGALRM, SIG_DFL);

    pclose(p1);

    return(0);
}

